/* jinx.c v.1.0c Aztec C ovr         */
/* Written by Wayne Parham           */ 
/* 09/01/2022                        */


#include "stdio.h"
#include "jinx.h"


void initialize() {
   setmem( buffer, SCREENWIDTH, 0 );
   setmem( visited, sizeof(visited[0])*(LastLocation+1), 0 );
   setmem( Inventory, sizeof(Inventory[0])*(LastItem+1), 0 );
#ifdef ENBLSAVE
   setmem( &saveState, sizeof(saveState), 0 );
#endif
   initFormats();
   initRes1Responses();
   initRes2Responses();
   initItmNames();
   initItm1Details();
   initItm2Details();
   initItmArrays();
   initLocDescriptions();
   initLD1LocDetails();
   initLD2LocDetails();
   initLocations();
   initDirections();
   initMessages();
   initMap1();
   initMap2();

   WorldState = 1;
   turnsPlayed = 0;
   hoursPassed = 0;
   minutesPassed = 0;
   turnCandle = 0;
   turnBell = 0;
   turnBook = 0;
   wraithEncounters = 0;
   wraithStrength = INITWRSTRENGTH;
   wraithAgitation = INITWRAGITATION;
   turnsToAgitate = DEFAULTAGITATEFREQUENCY;

   Move[30][North] = 0;                                              /* 3017 Front Door */
   Move[37][North]  = 0;                                             /* 3017 Shelter    */
   Move[38][South] = 0;
   Move[78][Down] = 0;                                               /* 4735 Garage     */
   Move[140][Up] = 0;
   Move[121][Down] = 0;                                              /* Forest Grate    */
   Move[142][Up] = 0;

}


int strncasecmp(s1, s2, length)                                      /* String comparison */
    char* s1;
    char* s2;
    int length;
{
    char* c1;
    char* c2;
    int   l;
    int   r;

    c1 = s1;
    c2 = s2;
    l  = 0;
    r  = 0;

    while( (r == 0) && (l < length) && (*c2 != '\0') ) {
       r = *c1 - *c2;
       c1++;
       c2++;
       l++;
    }

    return( r );
}


void clearScreen() {                                                 /* Clear screen */
   int i;

   for( i = 0; i < SCREENHEIGHT; i++ )
      printf( Cr );
}


void timeElapsed( minutes ) {                                        /* Record passage of time */
   if( minutes > 0 ) {
      minutesPassed += minutes;
      if( minutesPassed > 60 ) {
         hoursPassed++;
         minutesPassed -= 60;
      }
   } else {
      minutesPassed -= minutes;
      if( (minutesPassed < 0) && (hoursPassed > 0) ) {
         hoursPassed--;
      }
   }
}


void bufferToLower() {                                               /* Convert input buffer to lowercase */
   int i;

   for( i=0; buffer[i]; i++ )
      buffer[i] = tolower( buffer[i] );
}


void setState( state ) int state; {                                  /* Sets WorldState bit */
   WorldState = WorldState | (1 << state);
} 


void clearState( state ) int state; {                                /* Clears WorldState bit */
   WorldState = WorldState & ~(1 << state);
}


int getState( state ) int state; {                                   /* Get WorldState bit */
   if( ( (WorldState) >> (state) ) & 1 )
      return( TRUE );
   else
      return( FALSE );
}


int running() {                                                      /* Check to see if game over */
   return( getState(0) );
}


int rand() {                                                         /* Get random number */
    random = random * 1103515245 + 12345;
    return( random / 65536 ) % 100;                                  /* between 0 and 99  */
}


void srand( seed ) unsigned int seed; {
    random = seed;
}


/* ======================================== MAIN ======================================== */ 


int main() {
   initialize();
   clearScreen();
   printf( s, introText );
   look();

   while( running() ) {
      prompt();
      playerAction();
      checkFate();
   }

   return( NORMAL );
}
